# --------------------------------------------------------
#
# All categories
#

CREATE TABLE `categories` (
  `cat_id` int(11) NOT NULL auto_increment,
  `cat_name` varchar(255) NOT NULL default '',
  `parent_id` int(11) NOT NULL default '0',
  PRIMARY KEY  (`cat_id`)
);

# --------------------------------------------------------
#
# List of all keywords used to describe resource
#

CREATE TABLE `keywords` (
  `key_id` int(11) NOT NULL auto_increment,
  `key_name` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`key_id`),
  UNIQUE KEY `key_name` (`key_name`)
);

# --------------------------------------------------------
#
# List of resources
#

CREATE TABLE `resources` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL default '',
  `native_name` varchar(255) NOT NULL default '',
  `author` varchar(255) NOT NULL default '',
  `date` date NOT NULL default '0000-00-00',
  `file_path` varchar(255) NOT NULL default '',
  `remarks` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`id`)
);

# --------------------------------------------------------
#
# Connection between resource and cateogries
#

CREATE TABLE `resources_categories` (
  `res_id` int(11) NOT NULL default '0',
  `cat_id` int(11) NOT NULL default '0'
);

# --------------------------------------------------------
#
# Connection between resource and keywords
#

CREATE TABLE `resources_keywords` (
  `res_id` int(11) NOT NULL default '0',
  `key_id` int(11) NOT NULL default '0'
);


# --------------------------------------------------------
#
# Resource versions
#

CREATE TABLE `versions` (
  `v_id` int(11) NOT NULL auto_increment,
  `res_id` int(11) NOT NULL default '0',
  `version` int(11) NOT NULL default '0',
  `source_version` int(11) NOT NULL default '0',
  `filename` varchar(255) NOT NULL default '',
  `is_preview` tinyint(4) NOT NULL default '0',
  `is_hidden` tinyint(4) NOT NULL default '0',
  PRIMARY KEY  (`v_id`)
);


# --------------------------------------------------------
#
# Definition of relation
#
CREATE TABLE `relation_definition` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(50) NOT NULL default '',
  PRIMARY KEY  (`id`),
  KEY `name` (`name`)
);


-- --------------------------------------------------------
-- 
-- Resource relations
-- 
CREATE TABLE `resources_relations` (
  `source_id` int(11) NOT NULL default '0',
  `relation_id` int(11) NOT NULL default '0',
  `dest_id` int(11) NOT NULL default '0'
);


# --------------------------------------------------------
#
# collection of user resources
#
CREATE TABLE `user_collection` (
  `id` int(11) NOT NULL auto_increment,
  `user` varchar(50) NOT NULL default '',
  `res_id` int(11) NOT NULL default '0',
  PRIMARY KEY  (`id`)
);


# --------------------------------------------------------
#
# Labels
#

CREATE TABLE `labels` (
  `id` varchar(25) NOT NULL default '',
  `language` varchar(10) NOT NULL default '',
  `label` varchar(100) NOT NULL default '',
  KEY `id` (`id`)
)


# --------------------------------------------------------
#
# `user_collection` database structure
#

CREATE TABLE `user_collection` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `user` VARCHAR( 50 ) NOT NULL ,
  `res_id` INT NOT NULL ,
  PRIMARY KEY ( `id` )
);

# --------------------------------------------------------

#
# `users` database structure
#

CREATE TABLE `users` (
  `user_id` int(11) NOT NULL auto_increment,
  `name` varchar(100) NOT NULL default '',
  PRIMARY KEY  (`user_id`)
); 

# --------------------------------------------------------

#
# `permissions` database structure
#

CREATE TABLE `permissions` (
  `perm_id` int(11) NOT NULL auto_increment,
  `user_id` int(11) NOT NULL default '0',
  `object_id` int(11) NOT NULL default '0',
  `type_id` int(11) NOT NULL default '0',
  PRIMARY KEY  (`perm_id`)
); 

