<?php

/**
 * This class represents category
 */
class CCategory{
	
	//-------------------------------------------------------------------------
	/**
	 * Constructor
	 */
	public function __construct($database, $id){
		$this->db = $database;
		$this->id = $id;
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * Add categorie
	 * @return id of the newly added category
	 */
	public function addSubCategory($name){

		$id = $this->db->nextId('categories');
		$query = "INSERT INTO categories SET cat_id=$id, cat_name ='$name', parent_id = $this->id";
		$result = $this->db->query($query);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());

		return $id;
	}


	//-------------------------------------------------------------------------
	/**
	 * Add subcategories
	 */
	public function addSubCategories($names){

		$sql = "INSERT INTO categories SET cat_name =?, parent_id = $this->id";
		$sth = $this->db->prepare($sql);
		$result = $this->db->executeMultiple($sth, $names);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());

	}


	//-------------------------------------------------------------------------
	/**
	 * Get categories
	 * @return array with categories
	 */
	public function getSubCategories(){

		$categories = array();
		$query = "SELECT cat_id, cat_name FROM categories WHERE parent_id=$this->id";
		$result = $this->db->query($query);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());

		while($row = $result->fetchRow()) {
			$category['id'] = $row[0];
			$category['name'] = $row[1];
			array_push($categories,$category);
		}		
		return $categories;
	}


	//-------------------------------------------------------------------------
	/**
	 * Get all children. 
	 * This function check re
	 * @return array with categories
	 */
	public function getAllChildren(){

		$ids = array();
		$query = "SELECT cat_id FROM categories WHERE parent_id=$this->id";
		$result = $this->db->query($query);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") <br/>" . $result->getMessage());

		while($row = $result->fetchRow()) {
			$category = new CCategory($this->db, $row[0]);
			$ids = array_merge($ids, $category->getAllChildren());
			array_push($ids, $row[0]);
		}
				
		return $ids;
	}


	//-------------------------------------------------------------------------
	/**
	 * Get categories path
	 * @return array with categories
	 */
	public function getAllParents(){

		$categories = array();
		$parent = $this->id;
		while ($parent != 0) {
			$query = "SELECT cat_id, cat_name, parent_id FROM categories WHERE cat_id=$parent";

			$result = $this->db->query($query);
			if(DB::isError($result)) 
	    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());
			while($row = $result->fetchRow()) {
				if($row[0] != $this->id){
					$category['id'] = $row[0];
					$category['name'] = $row[1];
					$category['parent'] = $row[2];
					array_push($categories, $category);
				}
				$parent = $row[2];
			}
		}
		return array_reverse ($categories);
	}
	
	
	//-------------------------------------------------------------------------
	/**
	 * Get category name
	 * @return array with categories
	 */
	public function getName(){

		$name = "Top level category";
		
		if($this->id != 0){
			$query = "SELECT cat_name FROM categories WHERE cat_id=$this->id";
			$result = $this->db->query($query);
			if(DB::isError($result)) 
	    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());

			if($row = $result->fetchRow())
				$name = $row[0];
		} 
		
		return $name;
	}
	
	
	//-------------------------------------------------------------------------
	/**
	 * Get category parent
	 * @return array with categories
	 */
	public function getParent(){
			$query = "SELECT parent_id FROM categories WHERE cat_id=$this->id";
			$result = $this->db->query($query);
			$row = $result->fetchRow(); 
		return $row[0];
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Count all resources in this category 
	 * @return number of resources in this category
	 */
	public function resourceCount(){

		$sql = "SELECT count(*) FROM resources_categories WHERE cat_id=$this->id";
		$result = $this->db->query($sql);
		$row = $result->fetchRow();
		
		return $row[0];
	}


	//-------------------------------------------------------------------------
	/**
	 * Remove this category 
	 */
	public function remove(){

		$sql = "DELETE FROM categories WHERE cat_id=$this->id";
		$result = $this->db->query($sql);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());
	}


	//-------------------------------------------------------------------------
	/**
	 * Return all categories as list
	 * @return array with categories with full path
	 */
	public function getAllAsList(){
			
		$query = "SELECT cat_id, cat_name, parent_id FROM categories" . 
						 " ORDER BY cat_name";
		$result = $this->db->query($query);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") :" . $result->getMessage());
    	
    $categories = array();
		while($row = $result->fetchRow()){
			$category['id'] = $row[0];
			$category['name'] = $row[1];
			$category['parent'] = $row[2];
			
			array_push($categories, $category);
		}
		
		$tree = array();
		foreach($categories as $category){
		
			if($category['parent'] == 0){
				$list = array();
				$name = $category['name'];
				$this->makeList(&$categories, $category['id'], "", &$list);
				$item['id'] = $category['id'];
				$item['name'] = $category['name'];
				$item['values'] = $list;
				array_push($tree, $item);
			}
		}
		 
		return $tree;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Convert array with categories to list
	 * The loop for is nessesery because $category is passed by reference. 
	 * In this case foreach doesn't work
	 *
	 * @param $categories array with categories
	 * @param $parent_id parent id
	 * @return array as tree
	 */
	private function makeList(&$categories, $parent_id, $prefix, $tree){
	
		for($i = 0; $i < count($categories) ; $i++){
		
			$category = $categories[$i];
			if($category['parent'] == $parent_id){
				$id = $category['id'];
				if(strlen($prefix) > 0)
					$new_prefix = $prefix . " > " . $category['name'];
				else
					$new_prefix = $category['name'];
				$tree[$new_prefix] = $id;
				$this->makeList(&$categories, $id, $new_prefix, &$tree);
			}
		}
	}
	
	
	//---------------------------------------------------------------------------
	// Private members
	
	/** Database */
	private $db;
	/** category id */
	private $id;
}	
?>