<?php
/**
 * search for resources.
 * Restriction is array with fields:
 * type - "keyword" | "category"
 * data - restriction data
 *
 * $Id: cresourceresultset.php,v 1.14 2004/10/13 06:41:36 klangner Exp $
 */

require_once $CLASSES_ROOT . "odars/dao/ccategory.php";


class CResourceResultset{
	
	//-------------------------------------------------------------------------
	/**
	 * Constructor
	 */
	public function __construct($database, $restrictions){
		
		$this->db = $database;
		$this->expandRestrictions($restrictions);
	}	

	
	//-------------------------------------------------------------------------
	/**
	 * Get number of resources
	 */
	public function getSize(){

		$sql = $this->buildCountQuery();
				
		$result = $this->db->query($sql);
		$row = $result->fetchRow();
		
		return $row[0];
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * Get resources
	 */
	public function getResources($from, $count){

		$sql = $this->buildQuery();
		$sql .= " LIMIT $from, $count";
				
		$resources = array();
		$result = $this->db->query($sql);
		while($row = $result->fetchRow()){
			if(!isset($resources[$row[0]])){
				$resource['id'] = $row[0];
				$resource['name'] = $row[1];
				$resources[$row[0]] = $resource;
			}
		}
		
		return $resources;
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * Split restrictions to keywords and category
	 * Get all category id
	 */
	private function expandRestrictions($restrictions){

		$this->keywords = array();
		$this->categories = array();
		
		foreach($restrictions as $restriction){
			if($restriction['type'] == "keyword"){
				array_push($this->keywords, trim($restriction['data']));
			}else{
				array_push($this->categories, $restriction['data']);
			}
		}
		
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * build query
	 */
	private function buildCountQuery(){

		$keyword_count = count($this->keywords);
		$category_count = count($this->categories);
		
		$query = "SELECT count(*) FROM resources ";
		
		// Add keyword tables
		for($i = 0; $i < $keyword_count; $i++){
			$query .= ", resources_keywords AS rk$i, keywords AS k$i ";
		}
		// Add category tables
		for($i = 0; $i < $category_count; $i++){
			$query .= ", resources_categories AS rc$i, categories AS c$i ";
		}
		
		if($keyword_count + $category_count == 0)
			return $query;
		
		$query .= "WHERE ";
		$add_and = false;
		for($i = 0; $i < $keyword_count; $i++){
			if($add_and)
				$query .= " AND ";
			else
				$add_and = true;

			$query .= " resources.id=rk$i.res_id AND rk$i.key_id=k$i.key_id ";
		}
		// Add category tables
		for($i = 0; $i < $category_count; $i++){
			if($add_and)
				$query .= " AND ";
			else
				$add_and = true;

			$query .= " resources.id=rc$i.res_id AND rc$i.cat_id=c$i.cat_id ";
		}

		$i = 0;
		foreach($this->keywords as $keyword){
			$query .= " AND k$i.key_name='$keyword' ";
			$i ++;
		}
		
		$i = 0;
		foreach($this->categories as $category){
			$item = new CCategory($this->db, $category);
			$ids = $item->getAllChildren();

			$query .= " AND (c$i.cat_id=$category ";
			foreach($ids as $id){
				$query .= " OR c$i.cat_id=$id ";
			}
			$query .= " ) ";

			$i ++;
		}
		
		return $query;
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * build query
	 */
	private function buildQuery(){

		$keyword_count = count($this->keywords);
		$category_count = count($this->categories);
		
		$query = "SELECT resources.id, resources.name, resources.file_path " .
						 "FROM resources";
		
		// Add keyword tables
		for($i = 0; $i < $keyword_count; $i++){
			$query .= ", resources_keywords AS rk$i, keywords AS k$i ";
		}
		// Add category tables
		for($i = 0; $i < $category_count; $i++){
			$query .= ", resources_categories AS rc$i, categories AS c$i ";
		}
		
		if($keyword_count + $category_count == 0)
			return $query;
		
		$query .= "WHERE ";
		$add_and = false;
		for($i = 0; $i < $keyword_count; $i++){
			if($add_and)
				$query .= " AND ";
			else
				$add_and = true;

			$query .= " resources.id=rk$i.res_id AND rk$i.key_id=k$i.key_id ";
		}
		// Add category tables
		for($i = 0; $i < $category_count; $i++){
			if($add_and)
				$query .= " AND ";
			else
				$add_and = true;

			$query .= " resources.id=rc$i.res_id AND rc$i.cat_id=c$i.cat_id ";
		}

		$i = 0;
		foreach($this->keywords as $keyword){
			$query .= " AND k$i.key_name='$keyword' ";
			$i ++;
		}
		
		$i = 0;
		foreach($this->categories as $category){
			$item = new CCategory($this->db, $category);
			$ids = $item->getAllChildren();

			$query .= " AND (c$i.cat_id=$category ";
			foreach($ids as $id){
				$query .= " OR c$i.cat_id=$id ";
			}
			$query .= " ) ";

			$i ++;
		}
		
		return $query;
	}	
	
	
	//---------------------------------------------------------------------------
	// Private members
	/** Database */
	private $db;
	/** Restrictions to the result set */
	private	$keywords;
	private	$categories;
}	
?>