<?php
require_once $CLASSES_ROOT . "odars/dao/resource/cresource.php";

class CNewVersionAction{
	
	//-------------------------------------------------------------------------
	/**
	 * Constructor
	 * 	@database - database connection
	 *  @base_path - content path with resource files
	 *	@id - resource id
	 *  @files - array with file_description
	 *  @preview - preview file name
	 *
	 * file_description is another array with values:
	 *	name - filename
	 * 	source_version - source version or not set if new file
	 *  path - if new file or not set
	 *	hidden - file is hidden
	 */
	public function __construct($database, $base_path, $id, $files, $preview){
		$this->db = $database;
		$this->base_path = $base_path;
		$this->id = $id;
		$this->files = $files;
		$this->preview = $preview;
	}	

	
	//-------------------------------------------------------------------------
	/**
	 * Add version
	 */
	public function execute(){

		$resource = new CResource($this->db, $this->id); 
		$version = $resource->getLatestVersionNumber()+1;
		
		$path = $this->base_path.$resource->getPath() . "/$version/";
		$file_path = "";
		mkdir($path);
		$data = array();
		foreach($this->files as $file){
			
			$d[0] = $file['name'];
			if(isset($file['source_version'])){
				$d[1] = $file['source_version'];
				$file_path = $this->base_path.$resource->getPath() . "/$file[source_version]/";
				$file_path .= $file['name'];
			}
			else{
				$d[1] = $version;
				rename($file['path'], $path.$file['name']);
				$file_path = $path.$file['name'];
			}

			// create preview before moving file
			if($file['name'] === $this->preview){
				if($this->canMakePreview($file['name'])){
					$d[2] = false;
					$this->createPreview($file_path, $path.".preview", 320, 240);
					$d2[0] = ".preview.jpg";
					$d2[1] = $version;
					$d2[2] = true;
					$d2[3] = true;
					array_push($data, $d2);
				}
				else{
					$d[2] = true;
				}
			}
			else{
				$d[2] = false;
			}
			
			$d[3] = isset($file['hidden']);
			array_push($data, $d);
		}
		
		$query = "INSERT INTO versions SET " .
						 "res_id=$this->id, filename=?, " . 
						 "version=$version, source_version=?, is_preview=?, is_hidden=?";
		$sth = $this->db->prepare($query);
		$result = $this->db->executeMultiple($sth, $data);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());
    
	}
	
	
	//-------------------------------------------------------------------------
	/**
	 * check if we can make preview from given file
	 */
	private function canMakePreview($filename){
  	$ext = strtolower(substr($filename,strrpos($filename, ".")+1));
  	
  	if(	$ext == "jpg" || $ext == "jpeg" ||
  			$ext == "png" || $ext == "gif")
  	{
  		return true;
  	}
  	
  	return false;
	}
	
	
	//-------------------------------------------------------------------------
	/**
	 * create preview
	 * @param $input input file name
	 * @param $output output file name (without extension)
	 */
  private function createPreview($input, $output, $new_w, $new_h){
    
    $system=explode(".",$input);
    if(count($system) < 2){
    	echo $input;
    	return;
    }
    
    if (preg_match("/jpg|jpeg/",strtolower($system[1]))){
        $src_img=imagecreatefromjpeg($input);
    }
    else if (preg_match("/png/",strtolower($system[1]))){
        $src_img=imagecreatefrompng($input);
    }
    else if (preg_match("/gif/",strtolower($system[1]))){
        $src_img=imagecreatefromgif($input);
    }
    $input_x=imageSX($src_img);
    $input_y=imageSY($src_img);   

    $old_x=imageSX($src_img)/$new_w;
    $old_y=imageSY($src_img)/$new_h;

    if ($old_x >= $old_y) {
        $thumb_w=$new_w;
        $thumb_h=round(imageSY($src_img)/$old_x);
    } else {
        $thumb_w=round(imageSX($src_img)/$old_y);
        $thumb_h=$new_h;
    }

    $dst_img=ImageCreateTrueColor($thumb_w,$thumb_h);
    
		imagecopyresampled($dst_img,$src_img,0,0,0,0,$thumb_w,$thumb_h,$input_x,$input_y);

    imagejpeg($dst_img, $output . ".jpg");
    imagedestroy($dst_img);
    imagedestroy($src_img);
	}
	
	


	//---------------------------------------------------------------------------
	// Private members
	/** Database */
	private $db;
	/** Resource properties */
	private $id;
	/** file information */
	private $files;
	/** Preview file name */
	private $preview;
	/** Base file path */
	private $base_path;
}	
?>