<?php
require_once $CLASSES_ROOT . "odars/util/preview.php";

class CResource{
	
	//-------------------------------------------------------------------------
	/**
	 * Constructor
	 */
	public function __construct($database, $id){
		$this->db = $database;
		$this->id = $id;
		$this->exists = false;
		
		$this->init();
	}	

	
	//-------------------------------------------------------------------------
	/**
	 * Return next resource id
	 */
	public static function getNextID($database){

		return $database->nextId('resources');				
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * Create new resource
	 */
	public static function createNew($database, $base_path, $data, $temp_path){

		$path = CResource::createPath($base_path, $data['id']);
		$english_name = addslashes($data['english_name']);
		$native_name = addslashes($data['native_name']);
		$author = addslashes($data['author']);
		$remarks = addslashes($data['remarks']);
		$query = "INSERT INTO resources SET id=$data[id], name='$english_name', " . 
							"native_name='$native_name', author='$author', date=NOW(), " .
							"file_path='$path', remarks='$remarks'";
		
		$result = $database->query($query);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());
							
		$resource = new CResource($database, $data['id']);

		$files = array();
		$path = $base_path . $resource->getPath() . "/1/";
		mkdir($path);
		foreach($data['files'] as $filename){
			$file['name'] = $filename;
			$file['source_version'] = 1;
			array_push($files, $file);
			//rename($temp_path.$filename, $path.$filename);
		}
		
		$resource->createNewVersion(1, $files, $data['preview']);
		$resource->updateKeywords($data['keywords']);
		$resource->updateCategories($data['categories']);
		
		return $resource;
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * Init resource data
	 */
	public function getID(){

		return $this->id;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Check if resource exists
	 */
	public function exists(){

		return $this->exists;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Get resource name
	 */
	public function getName(){

		return $this->name;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Get resource native name
	 */
	public function getNativeName(){

		return $this->native_name;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Get resouce author
	 */
	public function getAuthor(){

		return $this->author;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Get creation date
	 */
	public function getCreationDate(){

		return $this->entered;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Get remarks
	 */
	public function getRemarks(){

		return $this->remarks;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * get resource path
	 */
	public function getPath(){

		return $this->path;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Get preview filename
	 */
	public function getPreviewFileName($version = 0){

		$file_name = "";
		
		if($version == 0){
			$query = "SELECT filename, source_version FROM versions " . 
							 "WHERE res_id=$this->id AND is_preview=1 " .
							 "ORDER BY version DESC LIMIT 1";
		}
		else{
			$query = "SELECT filename, source_version FROM versions " . 
							 "WHERE res_id=$this->id AND is_preview=1 AND version<=$version " .
							 "ORDER BY version DESC LIMIT 1";
		}

		$result = $this->db->query($query);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());

		if($row = $result->fetchRow()){
			$file_name = $this->getPath() . "/$row[1]/" . $row[0];
		}
		
		return $file_name;
	}

	
	//-------------------------------------------------------------------------
	/**
	 * Update resource categories
	 * $categories array with categories
	 */
	public function updateCategories($categories){

		// remove old categories
		$query = "DELETE FROM resources_categories WHERE res_id=$this->id";
		$result = $this->db->query($query);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());

		// add new categories
		$sth = $this->db->prepare("INSERT INTO resources_categories SET res_id=$this->id, cat_id=?");
		$result = $this->db->executeMultiple($sth, $categories);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());
		
	}


	//-------------------------------------------------------------------------
	/**
	 * Update keywords
	 */
	public function updateKeywords($keywords){
		
		$keys = array();
		$keys = split(",", $keywords);

		// Remove previous keywords
		$query = "DELETE FROM resources_keywords WHERE res_id = $this->id";
		$result = $this->db->query($query);

		// Add new keywords
		$query2 = "SELECT  key_id,key_name FROM keywords WHERE ";
		foreach($keys as $k){
			$k=trim(addslashes($k));
			$query = "INSERT INTO keywords SET key_name ='$k'";
			$result = $this->db->query($query);
			$query2 .= " key_name='".$k."' OR";
		}
		$query2 = substr($query2,0,-2);
		$result2 = $this->db->query($query2);
		
		while($row = $result2->fetchRow()) {
			$query3 = "INSERT INTO resources_keywords SET key_id =$row[0], res_id=$this->id";
			$result3 = $this->db->query($query3);
		}
	}


	//-------------------------------------------------------------------------
	/**
	 * Remove this resource
	 */
	public function removeResource(){

		$query = "DELETE FROM resources WHERE id =$this->id ";
		$result = $this->db->query($query);
		$query2 = "DELETE FROM resources_categories WHERE res_id =$this->id ";
		$result2 = $this->db->query($query2);
		$query3 = "DELETE FROM resources_keyword WHERE res_id =$this->id ";
		$result3 = $this->db->query($query3);
		$query4 = "DELETE FROM versions WHERE res_id =$this->id ";
		$result4 = $this->db->query($query4);
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * Get resource keywords
	 * @return array with keywords for resource
	 */
	public function getKeywords(){

		$keys = array();
		$query = "SELECT key_name FROM keywords as t1, resources_keywords as t2 where t1.key_id = t2.key_id and t2.res_id = $this->id";
		$result = $this->db->query($query);
		while($row = $result->fetchRow()) {
			array_push($keys,$row[0]);
		}
		return $keys;
	}

	//-------------------------------------------------------------------------
	/**
	 * Get resource categories
	 * @return array with category id
	 */
	public function getCategories(){

		$keys = array();
		$query = "SELECT cat_id FROM resources_categories WHERE res_id = $this->id";
		$result = $this->db->query($query);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());
		
		while($row = $result->fetchRow()) {
			array_push($keys,$row[0]);
		}
		
		return $keys;
	}

	//-------------------------------------------------------------------------
	/**
	 * Update resource
	 * @
	 */
	public function updateResource($name, $native_name, $author, $remarks){

		$name = addslashes($name);
		$native_name = addslashes($native_name);
		$author = addslashes($author);
		$remarks = addslashes($remarks);
		$query = "UPDATE resources SET name='$name', native_name='$native_name', " . 
						 "author='$author',remarks = '$remarks' WHERE id = $this->id";
		$result = $this->db->query($query);
	}
	

	//-------------------------------------------------------------------------
	/**
	 * Add version
	 */
	public function createNewVersion($number, $files, $preview){

		$data = array();
		foreach($files as $file){
			$d[0] = $file['name'];
			$d[1] = $file['source_version'];
			$d[2] = ($file['name'] === $preview);
			$d[3] = isset($file['hidden']);
			array_push($data, $d);
		}
		
		$query = "INSERT INTO versions SET " .
						 "res_id=$this->id, filename=?, " . 
						 "version=$number, source_version=?, is_preview=?, is_hidden=?";
		$sth = $this->db->prepare($query);
		$result = $this->db->executeMultiple($sth, $data);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());
	}


	//-------------------------------------------------------------------------
	/**
	 * Get files list from version
	 * @return array files 
	 */
	public function getVersionFiles($version){

		$mfiles = array();
		$query = "SELECT v_id, filename, version, source_version FROM versions " .
							"WHERE res_id = $this->id AND version=$version AND is_hidden=0";
							
		$result = $this->db->query($query);
		while($row = $result->fetchRow()) {
			$file['id'] = $row[0];
			$file['name'] = $row[1];
			$file['source_version'] = $row[3];
			$file['path'] = $this->path."/".$row[3];
			array_push($mfiles,$file);
		}
		
		return $mfiles;
	}
	
		//-------------------------------------------------------------------------
	/**
	 * Remove from versions
	 */
	public function removeFromVersions($v_id){

		$query = "DELETE FROM versions WHERE v_id =$v_id ";
		$result = $this->db->query($query);
	}
	
	
	//-------------------------------------------------------------------------
	/**
	 * Remove from version
	 */
	public function removeFromVersion($filename,$version){

		$query = "DELETE FROM versions WHERE res_id =$this->id " . 
						 "AND filename='$filename'  AND version = '$version' ";
		$result = $this->db->query($query);
	}


	//-------------------------------------------------------------------------
	/**
	 * Get version number
	 * @return version number 
	 */
	public function getLatestVersionNumber(){
		
		$query = "SELECT max(version) FROM versions WHERE res_id = $this->id";
		$result = $this->db->query($query);
		if(DB::isError($result)) 
    	die(__FILE__ . " (" . __LINE__ . ") " . $result->getMessage());

		$row = $result->fetchRow();
		
		if($row[0] != "")
			return $row[0];
		else			
			return 0;
	}


	//-------------------------------------------------------------------------
	/**
	 * Create resource directory
	 */
	public static function createPath($base_path, $id){

		$path = "a".rand(0,99);
		if(!file_exists($base_path.$path))
			mkdir($base_path.$path);

		$path .= "/b".rand(0,99);
		if(!file_exists($base_path.$path))
			mkdir($base_path.$path);
		
		$path .= "/c".rand(0,99);
		if(!file_exists($base_path.$path))
			mkdir($base_path.$path);
		
		$path .= "/$id";
		if(!file_exists($base_path.$path))
			mkdir($base_path.$path);
		
		return $path;
	}	
	
	
	//-------------------------------------------------------------------------
	/**
	 * Init resource data
	 */
	private function init(){

		$query = "SELECT name, native_name, author, file_path, date, remarks " . 
						 "FROM resources where id = $this->id";
		$result = $this->db->query($query);
		
		if($row = $result->fetchRow()){
			$this->name = $row[0];
			$this->native_name = $row[1];
			$this->author = $row[2];
			$this->path = $row[3];
			$this->entered = $row[4];
			$this->remarks = $row[5];
			$this->exists = true;
		}
	}

	
	//---------------------------------------------------------------------------
	// Private members
	/** Database */
	private $db;
	/** True if resource exists */
	private $exists;
	/** Resource properties */
	private $id;
	private $name;
	private $native_name;
	private $author;
	private $path;
	private $entered;
	private $remarks;
}	
?>